/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details:
 *
 * Copyright (C) 2008 Novell, Inc.
 * Copyright (C) 2009 - 2010 Red Hat, Inc.
 * Copyright (C) 2011 Novatel Wireless, Inc.
 */

#ifndef MM_MODEM_LTE_NETWORK_H
#define MM_MODEM_LTE_NETWORK_H

#include <mm-modem.h>
#include <mm-modem-lte.h>

#define MM_MODEM_LTE_NETWORK_DBUS_INTERFACE "org.freedesktop.ModemManager.Modem.Lte.Network"

#define MM_TYPE_MODEM_LTE_NETWORK      (mm_modem_lte_network_get_type ())
#define MM_MODEM_LTE_NETWORK(obj)      (G_TYPE_CHECK_INSTANCE_CAST ((obj), MM_TYPE_MODEM_LTE_NETWORK, MMModemLteNetwork))
#define MM_IS_MODEM_LTE_NETWORK(obj)   (G_TYPE_CHECK_INSTANCE_TYPE ((obj), MM_TYPE_MODEM_LTE_NETWORK))
#define MM_MODEM_LTE_NETWORK_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), MM_TYPE_MODEM_LTE_NETWORK, MMModemLteNetwork))

#define MM_MODEM_LTE_NETWORK_ALLOWED_MODE      "allowed-mode"
#define MM_MODEM_LTE_NETWORK_ACCESS_TECHNOLOGY "access-technology"

typedef enum {
    MM_MODEM_LTE_NETWORK_PROP_FIRST = 0x1200,

    MM_MODEM_LTE_NETWORK_PROP_ALLOWED_MODE = MM_MODEM_LTE_NETWORK_PROP_FIRST,
    MM_MODEM_LTE_NETWORK_PROP_ACCESS_TECHNOLOGY,
} MMModemLteNetworkProp;

typedef enum {
    MM_MODEM_LTE_NETWORK_REG_STATUS_IDLE = 0,
    MM_MODEM_LTE_NETWORK_REG_STATUS_HOME = 1,
    MM_MODEM_LTE_NETWORK_REG_STATUS_SEARCHING = 2,
    MM_MODEM_LTE_NETWORK_REG_STATUS_DENIED = 3,
    MM_MODEM_LTE_NETWORK_REG_STATUS_UNKNOWN = 4,
    MM_MODEM_LTE_NETWORK_REG_STATUS_ROAMING = 5
} MMModemLteNetworkRegStatus;

typedef struct _MMModemLteNetwork MMModemLteNetwork;

typedef void (*MMModemLteNetworkScanFn) (MMModemLteNetwork *self,
                                         GPtrArray *results,
                                         GError *error,
                                         gpointer user_data);

typedef void (*MMModemLteNetworkRegInfoFn) (MMModemLteNetwork *self,
                                            MMModemLteNetworkRegStatus status,
                                            const char *oper_code,
                                            const char *oper_name,
                                            GError *error,
                                            gpointer user_data);

struct _MMModemLteNetwork {
    GTypeInterface g_iface;

    /* Methods */
    /* 'register' is a reserved word */
    void (*do_register) (MMModemLteNetwork *self,
                         const char *network_id,
                         MMModemFn callback,
                         gpointer user_data);

    void (*scan) (MMModemLteNetwork *self,
                  MMModemLteNetworkScanFn callback,
                  gpointer user_data);

    void (*set_apn) (MMModemLteNetwork *self,
                     const char *apn,
                     MMModemFn callback,
                     gpointer user_data);

    void (*get_signal_quality) (MMModemLteNetwork *self,
                                MMModemUIntFn callback,
                                gpointer user_data);

    void (*set_band) (MMModemLteNetwork *self,
                      MMModemLteBand band,
                      MMModemFn callback,
                      gpointer user_data);

    void (*get_band) (MMModemLteNetwork *self,
                      MMModemUIntFn callback,
                      gpointer user_data);

    void (*set_allowed_mode) (MMModemLteNetwork *self,
                              MMModemLteAllowedMode mode,
                              MMModemFn callback,
                              gpointer user_data);

    void (*get_registration_info) (MMModemLteNetwork *self,
                                   MMModemLteNetworkRegInfoFn callback,
                                   gpointer user_data);

    /* Signals */
    void (*signal_quality) (MMModemLteNetwork *self,
                            guint32 quality);

    void (*registration_info) (MMModemLteNetwork *self,
                               MMModemLteNetworkRegStatus status,
                               const char *open_code,
                               const char *oper_name);
};

GType mm_modem_lte_network_get_type (void);

void mm_modem_lte_network_register (MMModemLteNetwork *self,
                                    const char *network_id,
                                    MMModemFn callback,
                                    gpointer user_data);

void mm_modem_lte_network_scan (MMModemLteNetwork *self,
                                MMModemLteNetworkScanFn callback,
                                gpointer user_data);

void mm_modem_lte_network_set_apn (MMModemLteNetwork *self,
                                   const char *apn,
                                   MMModemFn callback,
                                   gpointer user_data);

void mm_modem_lte_network_get_signal_quality (MMModemLteNetwork *self,
                                              MMModemUIntFn callback,
                                              gpointer user_data);

void mm_modem_lte_network_set_band (MMModemLteNetwork *self,
                                    MMModemLteBand band,
                                    MMModemFn callback,
                                    gpointer user_data);

void mm_modem_lte_network_get_band (MMModemLteNetwork *self,
                                    MMModemUIntFn callback,
                                    gpointer user_data);

void mm_modem_lte_network_get_registration_info (MMModemLteNetwork *self,
                                                 MMModemLteNetworkRegInfoFn callback,
                                                 gpointer user_data);

/* Protected */

void mm_modem_lte_network_signal_quality (MMModemLteNetwork *self,
                                          guint32 quality);

void mm_modem_lte_network_set_allowed_mode (MMModemLteNetwork *self,
                                            MMModemLteAllowedMode mode,
                                            MMModemFn callback,
                                            gpointer user_data);

void mm_modem_lte_network_registration_info (MMModemLteNetwork *self,
                                             MMModemLteNetworkRegStatus status,
                                             const char *oper_code,
                                             const char *oper_name);

#endif /* MM_MODEM_LTE_NETWORK_H */
